Package: sqldf
Version: 0.4-12
Date: 2026-01-29
Title: Manipulate R Data Frames Using SQL
Authors@R: person(given = "G.",
                  family = "Grothendieck",
                  role = c("aut", "cre"),
                  email = "ggrothendieck@gmail.com")
Description: The sqldf() function is typically passed a single argument which 
	is an SQL select statement where the table names are ordinary R data 
	frame names.  sqldf() transparently sets up a database, imports the 
	data frames into that database, performs the SQL select or other
	statement and returns the result using a heuristic to determine which 
	class to assign to each column of the returned data frame.  The sqldf() 
	or read.csv.sql() functions can also be used to read filtered files 
	into R even if the original files are larger than R itself can handle.
	'RSQLite', 'RH2', 'RMySQL' and 'RPostgreSQL' backends are supported.
ByteCompile: true
Depends: R (>= 3.1.0), gsubfn (>= 0.6), proto, RSQLite
Suggests: RH2, RMySQL, RPostgreSQL, svUnit, tcltk, MASS
Imports: DBI, chron
License: GPL-2
BugReports: https://github.com/ggrothendieck/sqldf/issues
URL: https://github.com/ggrothendieck/sqldf
NeedsCompilation: no
Packaged: 2026-01-30 02:30:02 UTC; louis
Author: G. Grothendieck [aut, cre]
Maintainer: G. Grothendieck <ggrothendieck@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-30 06:10:09 UTC
Built: R 4.6.0; ; 2026-04-26 02:50:37 UTC; unix
